<?php

/**
 * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
 * Class Shopware_Plugins_Backend_AduinkAduInkasso_Bootstrap
 */
class Shopware_Plugins_Backend_AduinkAduInkasso_Bootstrap extends Shopware_Components_Plugin_Bootstrap
{
    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return array
     */
    public function getCapabilities()
    {
        return array(
            'install' => true,
            'enable' => true,
            'update' => true // seit 1.1. verfügbar
        );
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return string
     */
    public function getLabel()
    {
        return 'ADU-Inkasso';
    }

    /**
     * aktuelle Version
     * 1.0.0 Original
     * 1.1.0 vom 11.12.2014
     * 1.1.1 vom 22.12.2014
     * 1.1.2 vom 19.06.2015
     * 1.1.3 vom 05.08.2015
     * 1.1.4 vom 02.09.2015
     * @return string
     */
    public function getVersion()
    {
        return "1.2.10";
    }

    /**
     * UpdateMethode
     * @param string $oldVersion
     * @return bool|void
     */
    public function update($oldVersion)
    {
        switch ($oldVersion) {
            case '1.0.0':
            case '1.1.0': // keine weiteren Änderungen notwendig
                $this->updateOneZeroZero();
                break;
            default:
            	$this->installPluginFormFields();
                $this->updateDB();
        }
        
        return true;
    }

    /**
     * UpdateFunktion für 1.0.0 auf 1.1.0
     */
    private function updateOneZeroZero(\Shopware\Models\Menu\Menu $menuParent = null)
    {

        $form = $this->Form();

        if (!$menuParent) {
            // Sollte wir keinen MenuParent übergeben bekommen, suchen wir ADU-Inkasso
            // Dieser ist bei einer Direktinstallation noch nicht in der Datenbank und kann daher so nicht gefunden werden.
            $menuParent = $this->Menu()->findOneBy('label', 'ADU-Inkasso');
        }

        /*
         * Neues Formularfeld
         */
        $form->setElement(
            'text',
            'frontend_score_fehlermeldung',
            array(
                'label' => 'Fehlermeldung Bonitätsprüfung Frontend',
                'required' => true,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => 'Die von Ihnen gewählte Zahlungsart steht nicht zur Verfügung. Bitte ändern Sie Ihre Zahlungsart.',
                'description' => 'Fehlermeldung die im Frontend angezeigt wird, wenn der Benutzer die Bonitätsprüfung nicht besteht.'
            )
        );

        /*
         * Formularfeld, Abfrage ob wir $smarty.block.child nutzen sollen
         */

        $form->setElement(
            'select',
            'frontend_use_smarty_block_child',
            array(
                'label' => 'Frontend Fehlermeldung Smarty Block Child nutzen',
                'description' => 'Template Erweiterung im Frontend bei Ablehnungsfehlermeldung der Zahlart. Entweder wird die Fehlermeldung via {block name="frontend_index_content" prepend} oder via {$smarty.block.child} eingefügt.',
                'store' => array(
                    array(0, 'Nein block prepend nutzen'),
                    array(1, 'Ja $smarty.block.child nutzen')
                ),
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => 0
            )
        );

        /*
         * Zusätzlicher Menüpunkt für Creditstandsabfrage
         */
        if ($menuParent) {
            $this->createMenuItem(
                array(
                    'label' => 'Creditstand',
                    'controller' => 'AduInkassoCreditstand',
                    'class' => 'sprite-adu-inkasso-logo',
                    'action' => 'Index',
                    'active' => 1,
                    'parent' => $menuParent
                    // Parent wird AduInkasso
                )
            );
            $this->createMenuItem(
                array(
                    'label' => 'Aktenübersicht',
                    'controller' => 'AduInkasso',
                    'class' => 'sprite-adu-inkasso-logo',
                    'action' => 'Index',
                    'active' => 1,
                    'parent' => $menuParent
                    // Kunden kommt 2x vor, daher empfiehlt sich nicht danach zu suchen
                )
            );
            $this->createMenuItem(
                array(
                        'label' => 'Auskünfte',
                        'controller' => 'AduInkassoAuskuenfte',
                        'class' => 'sprite-adu-inkasso-logo',
                        'action' => 'Index',
                        'active' => 1,
                        'parent' => $menuParent
                )
            );
        }

        /*
         * Neuer Controller AduInkassoCreditstand
         */
        $this->subscribeEvent(
            'Enlight_Controller_Dispatcher_ControllerPath_Backend_AduInkassoCreditstand',
            'onGetBackendControllerCreditstand'
        );


        // Widget installieren
        // erst ab Shopware 4.3.0
        $version = Shopware::VERSION;
        if (version_compare($version, '4.3.0', '>=')) {
            // http://wiki.shopware.com/Eigenes-Widget-erstellen_detail_1735.html#Controller
            $this->createWidget('aduink-creditstand');
            $this->registerController('Backend', 'AduinkCreditstandWidget');
        }


    }


    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return array
     */
    public function getInfo()
    {
        $info = file_get_contents($this->Path() . 'info.txt');

        return array(
            'author' => 'ADU-Inkasso',
            'supplier' => 'ADU-Inkasso',
            'version' => $this->getVersion(),
            'copyright' => 'Copyright (c) 2015, ADU-Inkasso',
            'label' => $this->getLabel(),
            'description' => $info,
            'support' => 'http://adu-inkasso.de/',
            'link' => 'http://adu-inkasso.de/',
            'revision' => '1'
        );
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     */
    private function clearCache()
    {
        Shopware()->Template()->clearAllCache();
        Shopware()->Template()->clearCompiledTemplate();
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return array|bool
     */
    public function install()
    {
        $version = Shopware::VERSION;
        if (version_compare($version, '4.1.2', '<')) {
            return array(
                'success' => false,
                'message' => 'Die Shopware Version muss mindestens 4.1.2 sein. Die Shopware Version ist ' . $version
            );
        }

        $this->installEvents();

        $this->installDatabase();

        $this->installAttributes();

        $menuItem = $this->installBackendNavigation();

        $this->installPluginFormFields();

        $this->installZahlungsstatus();

        $this->updateOneZeroZero($menuItem);

        $this->clearCache();
        
        $this->updateDB();

        return array(
            'success' => true,
            'cache' => array(
                'backend',
                'template',
                'proxy'
            )
        );
    }

    /**
     * Methode zum aktualisieren des Datenbankfeld der Tabelle s_core_customergroups_attributes 
     */
    private function updateDB(){
        $sql= "ALTER TABLE `s_core_customergroups_attributes` CHANGE `adu_inkasso_boni_check_exclude` `adu_inkasso_boni_check_exclude` TINYINT NULL DEFAULT NULL";
        Shopware()->Db()->query($sql);
        $sql= "ALTER TABLE `s_user_attributes` CHANGE `adu_inkasso_boni_check_exclude` `adu_inkasso_boni_check_exclude` TINYINT NULL DEFAULT NULL";
        Shopware()->Db()->query($sql);
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     */
    private function installAttributes()
    {
        /** @var \Shopware\Components\Model\ModelManager $models */
        $models = $this->Application()->Models();
        /*
         * BESTELLUNG
         */
        $models->addAttribute(
            's_order_attributes',
            'adu_inkasso',
            'aktenzeichen',
            'varchar(255)',
            true,
            null
        );
        /*
         * KUNDE
         */
        if (version_compare(Shopware::VERSION, '5.2', '<')) 
        {
            $models->addAttribute(
                's_user_attributes',
                'adu_inkasso',
                'boni_check_exclude',
                'tinyint(1)',
                true,
                0
            );
        }else{
            // Get the service
            $attributeService = Shopware()->Container()->get('shopware_attribute.crud_service');

            // Creation date
            $attributeService->update('s_user_attributes', 'adu_inkasso_boni_check_exclude', 'boolean', array('label' => 'Ausgenommen von ADU-Inkasso Bonitätsprüfung', 'translatable' => false, 'displayInBackend' => true, 'position' => 600) );

            // Rebuild attribute models
            $metaDataCache = Shopware()->Models()->getConfiguration()->getMetadataCacheImpl();
            $metaDataCache->deleteAll();
        }
        /*
         * Kundengruppen
         */
        $models->addAttribute(
            's_core_customergroups_attributes',
            'adu_inkasso',
            'boni_check_exclude',
            'tinyint(1)',
            true,
            NULL
        );

        $models->generateAttributeModels(
            array(
                's_order_attributes',
                's_user_attributes',
                's_core_customergroups_attributes'
            )
        );
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     */
    private function installBackendNavigation()
    {
        // -Modul
        return $this->createMenuItem(
            array(
                'label' => 'ADU-Inkasso',
                'controller' => 'AduInkasso',
                'class' => 'sprite-adu-inkasso-logo',
                'action' => 'Index',
                'active' => 1,
                'parent' => $this->Menu()->findOneBy(array('label' => 'Bestellungen'))->getParent()
                // Kunden kommt 2x vor, daher empfiehlt sich nicht danach zu suchen
            )
        );
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return string
     */
    public function getZahlungsstatusBeschreibung()
    {
        return 'ADU-Inkasso';
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return string
     */
    public function getZahlungsstatusInkassoBeendetBeschreibung()
    {
        return 'Inkasso beendet';
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     */
    private function installZahlungsstatus()
    {
        $models = Shopware()->Models()->getRepository('Shopware\Models\Order\Status');

        $zahlstati = array($this->getZahlungsstatusBeschreibung(), $this->getZahlungsstatusInkassoBeendetBeschreibung());

        foreach ($zahlstati as $zahlungsstatus) {

            $isOne = $models->findOneBy(array('description' => $zahlungsstatus));

            if (!$isOne) {

                $id = 50;

                $highest = $models->findBy(array(), array("id" => "DESC"), 1);
                if ($highest) {
                    $highest = current($highest);
                    $id = $highest->getId() + 1;
                }

                // Muss per query gelöst werden und nicht via Shopware()->Models() da die Primary-ID kein AutoIncrement hat.
                // Die Funktion setId bei Shopware\Models\Order\Status ist nicht vorhanden.
                Shopware()->Db()->query(
                    "" . "INSERT INTO `s_core_states` (`id`, `description`, `position`, `group`, `mail`) VALUES (" . $id . ", '" . $zahlungsstatus . "', '40', 'payment', '0')"
                );


            }
        }
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return bool
     */
    public function enable()
    {
        $this->clearCache();

        return true;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return bool
     */
    public function disable()
    {
        $this->removeAduRiskmanagementRulesFromDatabase();

        $this->clearCache();

        return true;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     */
    private function registerComponentsNamespace()
    {
        Shopware()->Loader()->registerNamespace(
            'Shopware_Components',
            $this->Path() . 'Components/'
        );
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     */
    private function removeAduRiskmanagementRulesFromDatabase()
    {
        $name = 'ADUINKASSO';

        $pdo = Shopware()->Db();

        $pdo->query(
            "" . "DELETE FROM `s_core_rulesets` WHERE `rule1` = ? OR `rule2` = ?",
            array(
                $name,
                $name
            )
        );
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     */
    private function installPluginFormFields()
    {
        $form = $this->Form();

        $repository = Shopware()->Models()->getRepository('Shopware\Models\Config\Form');
        /** @var \Shopware\Models\Config\Form $parentForm */
        $parentForm = $repository->findOneBy(array('name' => 'Interface'));
        $form->setParent(
            $parentForm
        );

        $form->setElement(
            'text',
            'schnittstelle_benutzer',
            array(
                'label' => 'Benutzername der Schnittstelle',
                'required' => true,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => '',
            )
        );
        $form->setElement(
            'text',
            'schnittstelle_passwort',
            array(
                'label' => 'Passwort der Schnittstelle',
                'required' => true,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => '',
            )
        );
        $form->setElement(
            'select',
            'checkout_pruefen',
            array(
                'label' => 'Bonitätsprüfung beim Checkout',
                'store' => array(
                    array(0, 'Keine Bonitätsprüfung durchführen.'),
                    array(1, 'Bonitätsprüfung durchführen')
                ),
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => 0
            )
        );
        /*
         * Default bei Firma
         */
        $defaultBoni = array();
        for ($i = 1; $i <= 6; $i += 0.1) {
            $defaultBoni[] = array($i, $i);
        }
        $form->setElement(
            'select',
            'checkout_default_boniwert_company',
            array(
                'label' => 'Default-Bonitätswert bei Firmen',
                'store' => $defaultBoni,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => 6
            )
        );

        /*
         * Default bei Fehler
         */
        $defaultBoni = array();
        for ($i = 1; $i <= 6; $i += 0.1) {
            $defaultBoni[] = array($i, $i);
        }
        $form->setElement(
            'select',
            'checkout_default_boniwert_error',
            array(
                'label' => 'Default-Bonitätswert bei Nichtermittlung',
                'store' => $defaultBoni,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => 6
            )
        );

        $old = array();
        for ($i = 1; $i <= 30; $i++) {
            $old[] = array($i, $i . ' Tag(e)');
        }
        
        // Extra
        $old[] = array(90, 90 . ' Tag(e)');
        $old[] = array(180, 180 . ' Tag(e)');
        $old[] = array(360, 360 . ' Tag(e)');
        
        $form->setElement(
            'select',
            'checkout_max_age',
            array(
                'label' => 'Höchstalter der Prüfung vor Neuprüfung',
                'store' => $old,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => 10
            )
        );

        /*
         * AMPELFARBE GRÜN
         */
        $store = array();
        for ($i = 1; $i <= 6; $i += 0.1) {
            $store[] = array($i, 'Score: 1 - ' . $i);
        }
        $form->setElement(
            'select',
            'score_light_green',
            array(
                'label' => 'Ampelfarbe: Grün',
                'store' => $store,
                'value' => 2.5,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'description' => 'Bürgel-Standard: 1.0 - 2.5, Statistisches Ausfallrisiko gering bis durchschnittlich'
            )
        );

        /*
         * AMPELFARBE GELB
         */
        $store = array();
        for ($i = 1; $i <= 6; $i += 0.1) {
            $store[] = array($i, 'Score: grün - ' . $i);
        }
        $form->setElement(
            'select',
            'score_light_yellow',
            array(
                'label' => 'Ampelfarbe: Gelb',
                'store' => $store,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => 2.9,
                'description' => 'Bürgel-Standard: 2.6 - 2.9, Statistisches Ausfallrisiko überdurchschnittlich'
            )
        );

        /*
         * AMPELFARBE ROT
         */

        $form->setElement(
            'select',
            'score_light_red',
            array(
                'label' => 'Ampelfarbe: ROT',
                'store' => array(array('6', 'gelb - 6.0')),
                'value' => 6,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'description' => 'Bürgel-Standard: 3.0 - 6.0, Es liegen negative Zahlungserfahrungen vor'
            )
        );


    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return bool
     */
    public function uninstall()
    {
        $this->removeAduRiskmanagementRulesFromDatabase();

        $this->clearCache();

        // $this->removeDatabaseTables();
        // $this->removeAttributes();

        return true;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     */
    private function installEvents()
    {
        $this->subscribeEvent(
            'Enlight_Controller_Dispatcher_ControllerPath_Backend_AduInkasso',
            'onGetBackendController'
        );
        
        $this->subscribeEvent(
                'Enlight_Controller_Dispatcher_ControllerPath_Backend_AduInkassoAuskuenfte',
                'onGetBackendControllerAuskuenfte'
        );
        
        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Backend_Customer',
            'onBackendCustomerPostDispatch'
        );
        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Backend_Order',
            'onBackendOrderPostDispatch'
        );

        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Backend_Index',
            'onBackendIndexPostDispatch'
        );

        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Backend_Config',
            'onBackendConfigPostDispatch'
        );

        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Backend_RiskManagement',
            'onBackendRiskManagementPostDispatch'
        );

        $this->subscribeEvent(
            'Enlight_Controller_Action_PreDispatch_Frontend_Checkout',
            'onFrontendCheckoutFinishActionPreDispatch'
        );
        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Frontend_Checkout',
            'onFrontendCheckoutFinishActionPostDispatch'
        );

        $this->subscribeEvent(
            'sAdmin::sManageRisks::replace',
            'sAdminsManageRisksReplaceHook'
        );

        $this->subscribeEvent(
            'Enlight_Bootstrap_InitResource_AduInkassoApi',
            'onInitMyComponent'
        );

    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param Enlight_Hook_HookArgs $arguments
     */
    public function sAdminsManageRisksReplaceHook(Enlight_Hook_HookArgs $arguments)
    {

        $paymentID = $arguments->get('paymentID');
        $basket = $arguments->get('basket');
        $user = $arguments->get('user');

        $subject = $arguments->getSubject();

        $return = $this->getBonitaetAdapter()->callsManageRisks($paymentID, $basket, $user, $subject);

        if (!$return) {
            $return = false;
        }

        $arguments->setReturn($return);

    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return string
     */
    public function onGetBackendController()
    {
        $this->Application()->Template()->addTemplateDir(
            $this->Path() . 'Views/'
        );

        return $this->Path() . 'Controllers/AduInkasso.php';
    }

    /**
     * Neu in 1.1.0
     * Controller für die Creditstandübersichten
     */
    public function onGetBackendControllerCreditstand()
    {
        $this->Application()->Template()->addTemplateDir(
            $this->Path() . 'Views/'
        );

        return $this->Path() . 'Controllers/AduInkassoCreditstand.php';
    }
    
    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return string
     */
    public function onGetBackendControllerAuskuenfte()
    {
        $this->Application()->Template()->addTemplateDir(
                $this->Path() . 'Views/'
                );
    
        return $this->Path() . 'Controllers/AduInkassoAuskuenfte.php';
    }
    
    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return Shopware_Components_AduInkassoBonitaet
     */
    public function getBonitaetAdapter()
    {
        $this->registerComponentsNamespace();

        return new Shopware_Components_AduInkassoBonitaet($this->Config(), $this->getApi());
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return Shopware_Components_AduInkassoApi
     */
    public function onInitMyComponent()
    {
        $this->registerComponentsNamespace();

        return new Shopware_Components_AduInkassoApi($this->Config()->schnittstelle_benutzer, $this->Config()->schnittstelle_passwort);
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param Enlight_Event_EventArgs $arguments
     */
    public function onFrontendCheckoutFinishActionPreDispatch(Enlight_Event_EventArgs $arguments)
    {

        /**@var $indexController Shopware_Controllers_Frontend_Index */
        $indexController = $arguments->getSubject();

        $request = $indexController->Request();
        $response = $indexController->Response();

        if (!$request->isDispatched()
            || $response->isException()
            || $request->getModuleName() != 'frontend'
            || $request->getControllerName() != 'checkout'
            || $request->getActionName() != 'finish'
            || !$indexController->View()->hasTemplate()
        ) {
            return;
        }

        $ok = $this->getBonitaetAdapter()->isZahlartOkOnCheckout();

        if (!$ok) {
            $indexController->forward('shippingPayment', 'checkout', 'frontend', array('aduinkassoPaymentError' => true));

            return;
        }

    }
            
    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return Shopware_Components_AduInkassoApi
     */
    public function getApi()
    {
        /** @var Shopware_Components_AduInkassoApi $api */
        $api = $this->Application()->AduInkassoApi();

        return $api;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param Enlight_Event_EventArgs $arguments
     */
    public function onFrontendCheckoutFinishActionPostDispatch(Enlight_Event_EventArgs $arguments)
    {

        /**@var $indexController Shopware_Controllers_Frontend_Index */
        $indexController = $arguments->getSubject();

        $request = $indexController->Request();
        $response = $indexController->Response();

        $action = $request->getActionName();

        $view = $indexController->View();

        if ($action !== 'shippingPayment' || !$request->isDispatched() ||
            $response->isException() || !$view->hasTemplate()
        ) {
            return;
        }

        if ($request->has('aduinkassoPaymentError')) {
            $view->assign('aduinkassoPaymentError', true);
            $view->assign('aduink_frontend_score_fehlermeldung', $this->Config()->frontend_score_fehlermeldung);
            $view->addTemplateDir($this->Path() . 'Views/');
            $shopware5 = version_compare(Shopware::VERSION,'5.0.0','>=');
            if ( $this->Config()->frontend_use_smarty_block_child == 0 ) {
                if ( $shopware5 ) {
                    $view->extendsTemplate('frontend/adu_inkasso/checkout5/confirm.tpl');
                } else {
                    $view->extendsTemplate('frontend/adu_inkasso/checkout/confirm.tpl');
                }
            } else {
                if ( $shopware5 ) {
                    $view->extendsTemplate('frontend/adu_inkasso/checkout5/confirm_smarty_block_child.tpl');
                } else {
                    $view->extendsTemplate('frontend/adu_inkasso/checkout/confirm_smarty_block_child.tpl');
                }
            }
        }
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     */
    public function afterInit()
    {
        $this->registerCustomModels();
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     */
    private function installDatabase()
    {
        $em = $this->Application()->Models();
        $tool = new \Doctrine\ORM\Tools\SchemaTool($em);

        $classes = array(
            $em->getClassMetadata('Shopware\CustomModels\AduInkasso\Bonitaetdatensatz')
        );

        try {
            $tool->createSchema($classes);
        } catch (\Doctrine\ORM\Tools\ToolsException $e) {
            // ignore
        }
        try {
            $tool->updateSchema($classes, true);
        } catch (\Doctrine\ORM\Tools\ToolsException $e) {
            // ignore
        }
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     */
    private function removeDatabaseTables()
    {
        $em = Shopware()->Models();
        $tool = new \Doctrine\ORM\Tools\SchemaTool($em);

        $classes = array(
            $em->getClassMetadata('Shopware\CustomModels\AduInkasso\Bonitaetdatensatz'),
        );

        try {
            $tool->dropSchema($classes);
        } catch (Exception $e) {
            // ignore
        }
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param Enlight_Event_EventArgs $args
     */
    public function onBackendConfigPostDispatch(Enlight_Event_EventArgs $args)
    {
        /** @var Enlight_Controller_Action $indexController */
        $indexController = $args->getSubject();
        /**@var $view Enlight_View_Default */
        $view = $indexController->View();
        /** @var Enlight_Controller_Request_RequestHttp $request */
        $request = $indexController->Request();
        /** @var Enlight_Controller_Response_ResponseHttp $response */
        $response = $indexController->Response();

        if (!$request->isDispatched() || $response->isException() || $request->getModuleName() != 'backend' || !$view->hasTemplate()) {
            return;
        }

        //if the controller action name equals "load" we have to load all application components
        if ($request->getActionName() === 'load') {
            // Add template directory
            $view->addTemplateDir(
                $this->Path() . 'Views/'
            );
            $view->extendsTemplate(
                'backend/config/view/adu_inkasso/customer_group/detail.js'
            );
            $view->extendsTemplate(
                'backend/config/model/adu_inkasso/form/customer_group.js'
            );
        }

        if ($request->getActionName() === 'getValues' &&
        (
                        $request->getQuery('name', false) == 'customerGroup' // Shopware 4
                        || $request->getQuery('_repositoryClass', false) == 'customerGroup' // Shopware 5
                    )
        ) {
            $data = $view->data;

            /** @var \Shopware\Components\Model\ModelManager $models */
            $models = $this->Application()->Models();
            /** @var \Shopware\Components\Model\ModelRepository $rep */
            $rep = $models->getRepository('Shopware\Models\Customer\Group');

            foreach ($data as &$customerGroup) {
                /** @var \Shopware\Models\Customer\Group $cg */
                $cg = $rep->find($customerGroup['id']);

                if (!$cg || !$cg->getAttribute()) {
                    $customerGroup['aduInkassoBoniCheckExclude'] = false;
                    continue;
                }
                $customerGroup['aduInkassoBoniCheckExclude'] = $cg->getAttribute()->getAduInkassoBoniCheckExclude();
            }
            $view->data = $data;
        }

        if ($request->getActionName() === 'saveValues' &&
            (
                $request->getQuery('name', false) == 'customerGroup' // Shopware 4
                || $request->getQuery('_repositoryClass', false) == 'customerGroup' // Shopware 5
            )
            ) {
            $setExclude = $request->getPost('aduInkassoBoniCheckExclude', false);
            $id = $request->getPost('id', false);
            if ($id) {
                /** @var \Shopware\Components\Model\ModelManager $models */
                $models = $this->Application()->Models();
                /** @var \Shopware\Components\Model\ModelRepository $rep */
                $rep = $models->getRepository('Shopware\Models\Customer\Group');
                /** @var \Shopware\Models\Customer\Group $cg */
                $cg = $rep->find($id);

                if ($cg) {
                    if (!$cg->getAttribute()) {
                        $attr = new \Shopware\Models\Attribute\CustomerGroup();
                        $this->Application()->Models()->persist($attr);
                        $cg->setAttribute($attr);
                    }
                    $attr = $cg->getAttribute();

                    $attr->setAduInkassoBoniCheckExclude($setExclude);

                    $models->flush();
                }

            }
        }


    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param Enlight_Event_EventArgs $args
     */
    public function onBackendCustomerPostDispatch(Enlight_Event_EventArgs $args)
    {
        /**@var $view Enlight_View_Default */
        $view = $args->getSubject()->View();

        // Add template directory
        $args->getSubject()->View()->addTemplateDir(
            $this->Path() . 'Views/'
        );

        if ($args->getRequest()->getActionName() == 'getList') {
            // Infos anhängen
            /** @var Array $data */
            $data = $view->data;
            foreach ($data as &$user) {
                $id = $user['id'];
                /** @var \Shopware\CustomModels\AduInkasso\Repository $rep */
                $rep = Shopware()->Models()->getRepository('Shopware\CustomModels\AduInkasso\Bonitaetdatensatz');
                $datensatz = $rep->getLastByCustomer($id);
                $user['adu_score_value'] = (!$datensatz) ? 0 : $datensatz->getScore();
                $user['adu_score_abfrage_datum'] = (!$datensatz) ? false : $datensatz->getAbfrageDatum();
            }
            $view->data = $data;
        }

        //if the controller action name equals "load" we have to load all application components
        if ($args->getRequest()->getActionName() === 'load') {
            $view->extendsTemplate(
                'backend/customer/view/detail/adu_inkasso/window.js'
            );
            $view->extendsTemplate(
                'backend/customer/view/list/adu_inkasso/list.js'
            );
            $view->extendsTemplate(
                'backend/customer/controller/adu_inkasso/main.js'
            );
            $view->extendsTemplate(
                'backend/customer/controller/adu_inkasso/detail.js'
            );
            $view->extendsTemplate(
                'backend/customer/model/adu_inkasso/list.js'
            );
            if (version_compare(Shopware::VERSION, '5.2', '<')) 
            {
                $view->extendsTemplate(
                    'backend/customer/view/detail/adu_inkasso/base.js'
                );
            }
            $view->extendsTemplate(
                'backend/customer/view/detail/adu_inkasso/additional.js'
            );
            $view->extendsTemplate(
                'backend/customer/model/adu_inkasso/attribute.js'
            );


        }
        if ($args->getRequest()->getActionName() === 'index') {
            $view->extendsTemplate(
                'backend/customer/controller/adu_inkasso/detail.js'
            );
            $view->extendsTemplate(
                'backend/customer/controller/adu_inkasso/main.js'
            );
        }

    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param Enlight_Event_EventArgs $args +
     */
    public function onBackendIndexPostDispatch(Enlight_Event_EventArgs $args)
    {
        /**@var $view Enlight_View_Default */
        $view = $args->getSubject()->View();
        $request = $args->getSubject()->Request();
        $response = $args->getSubject()->Response();

        if (!$request->isDispatched() || $response->isException() || $request->getModuleName() != 'backend' || !$view->hasTemplate()) {
            return;
        }

        // Add template directory
        $view->addTemplateDir(
            $this->Path() . 'Views/'
        );

        if ($args->getRequest()->getActionName() === 'load') {
            $view->assign('adu_inkasso_score_green', $this->Config()->score_light_green);
            $view->assign('adu_inkasso_score_yellow', $this->Config()->score_light_yellow);
            $view->extendsTemplate('backend/index/adu_inkasso/helper.js');

        }
        $view->extendsTemplate('backend/index/adu_inkasso/index.tpl');

        // Ab Version 1.1.0, Widget Erweiterung
        if (version_compare($this->getVersion(), '1.1.0', '>=')) {
            if ($args->getRequest()->getActionName() === 'index') {
                $view->extendsTemplate('backend/index/aduink_creditstand/app.js');
            }
        }
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param Enlight_Event_EventArgs $args
     */
    public function onBackendOrderPostDispatch(Enlight_Event_EventArgs $args)
    {
        /**@var $view Enlight_View_Default */
        $view = $args->getSubject()->View();

        // Add template directory
        $args->getSubject()->View()->addTemplateDir(
            $this->Path() . 'Views/'
        );

        if ($args->getRequest()->getActionName() == 'getList') {
            // Infos anhängen
            /** @var Array $data */
            $data = $view->data;
            $aktenzeichen = array();
            foreach ($data as &$order) {
                $id = $order['customerId'];
                /** @var \Shopware\CustomModels\AduInkasso\Repository $rep */
                $rep = Shopware()->Models()->getRepository('Shopware\CustomModels\AduInkasso\Bonitaetdatensatz');
                $datensatz = $rep->getLastByCustomer($id);
                $order['adu_score_value'] = (!$datensatz) ? 0 : $datensatz->getScore();
                $order['adu_score_abfrage_datum'] = (!$datensatz) ? false : $datensatz->getAbfrageDatum();

                if ($order['attribute']['aduInkassoAktenzeichen'] != "") {
                    $aktenzeichen[$order['id']] = $order['attribute']['aduInkassoAktenzeichen'];
                }
            }

            $view->data = $data;


            $api = $this->getApi();
            // Abfrage zusammenstellen
            if (count($aktenzeichen) > 0 && $api->testConnection()) {
                $response = $api->getStadium($aktenzeichen);
                if (!is_array($response)) {
                    return;
                }
                $view->aduResponse = $response;
                /** @var \Shopware\Components\Model\ModelManager $mm */
                $mm = $this->Application()->Models();
                $orderRep = $mm->getRepository('Shopware\Models\Order\Order');
                $zahlstatusRep = $mm->getRepository('Shopware\Models\Order\Status');
                $offen = $zahlstatusRep->findOneBy(
                    array('group' => 'payment', 'description' => 'Offen')
                );
                $aduStatus = $zahlstatusRep->findOneBy(
                    array('group' => 'payment', 'description' => $this->getZahlungsstatusBeschreibung())
                );
                $zahlungBeendetStatus = $zahlstatusRep->findOneBy(
                    array('group' => 'payment', 'description' => $this->getZahlungsstatusInkassoBeendetBeschreibung())
                );

                $change = false;

                foreach ($data as &$order) {
                    $id = $order['id'];

                    /** @var \Shopware\Models\Order\Order $orderObj */
                    $orderObj = $orderRep->find($id);

                    if ($orderObj) {

                        if (
                            (isset($aktenzeichen[$id]) && !isset($response[$aktenzeichen[$id]]))
                            ||
                            (!isset($aktenzeichen[$id]) && $order['paymentStatus']['id'] == $aduStatus->getId())
                        ) {
                            // Akte ist verknüpft, aber die Akte existiert nicht mehr.
                            // Zahlstatus geht zurück auf "offen"
                            if (!$orderObj->getAttribute()) {
                                $orderObj->setAttribute(array());
                            }
                            $orderObj->getAttribute()->setAduInkassoAktenzeichen(null);
                            $orderObj->setPaymentStatus($offen);
                            $change = true;
                        }

                        if ($order['paymentStatus']['id'] == $aduStatus->getId()) {
                            if (isset($aktenzeichen[$id]) && isset($response[$aktenzeichen[$id]]['einstellDatum'])) {
                                if (!empty($response[$aktenzeichen[$id]]['einstellDatum'])) {
                                    $orderObj->setPaymentStatus($zahlungBeendetStatus);
                                    $change = true;
                                }
                            }
                        }
                    }
                }

                if ($change) {
                    $mm->flush();
                    $args->getSubject()->forward('getList');

                    return;
                }
            }

        }

        //if the controller action name equals "load" we have to load all application components
        if ($args->getRequest()->getActionName() === 'load') {
            $view->extendsTemplate(
                'backend/order/view/list/adu_inkasso/list.js'
            );
            $view->extendsTemplate(
                'backend/order/model/adu_inkasso/order.js'
            );
        }
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param Enlight_Event_EventArgs $args
     */
    public function onBackendRiskManagementPostDispatch(Enlight_Event_EventArgs $args)
    {
        /**@var $view Enlight_View_Default */
        $view = $args->getSubject()->View();

        // Add template directory
        $args->getSubject()->View()->addTemplateDir(
            $this->Path() . 'Views/'
        );

        //if the controller action name equals "load" we have to load all application components
        if ($args->getRequest()->getActionName() === 'load') {
            $view->extendsTemplate(
                'backend/risk_management/store/adu_inkasso/risks.js'
            );
        }
    }


}