<?php

/**
 * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
 * Class Shopware_Components_AduInkassoApi
 */
class Shopware_Components_AduInkassoApi
{
    private $client = null;
    private $username;
    private $password;

    private $client_url = 'https://www.auskunft.inkassounternehmen.de/Webservice/adiservice.wsdl';

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param $username
     * @param $password
     */
    public function __construct($username, $password)
    {
        $this->username = $username;
        $this->password = $password;
    }


    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param $method
     * @param array $params
     * @param bool $unserialize
     * @return mixed
     */
    public function callClient($method, Array$params = array(), $unserialize = false)
    {
        $params = array_merge(
            array(
                $this->getUsername(),
                $this->getPassword()
            ),
            $params
        );
        $response = call_user_func_array(array($this->getClient(), $method), $params);

        if ($unserialize) {
            $response = unserialize($response);
        }

        return $response;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return null|SoapClient
     */
    private function getClient()
    {
        if ($this->client) {
            return $this->client;
        }
        ini_set('soap.wsdl_cache_enabled', 0);
        $options = array(
            'soap_version' => SOAP_1_1,
            'trace' => 1,
            'exceptions' => 1
        );

        $client = new SoapClient($this->client_url, $options);

        return $client;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return bool
     */
    public function testConnection()
    {
        return $this->callClient('testConnection') == 1;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return mixed
     */
    public function getPassword()
    {
        return $this->password;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return mixed
     */
    public function getUsername()
    {
        return $this->username;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param array $aktenzeichen
     * @return mixed
     */
    public function getStadium(Array$aktenzeichen)
    {
        return $this->callClient('getStadium', array(implode(',', $aktenzeichen)), true);
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param \Shopware\Models\Customer\Customer $customer
     * @return bool|\Shopware\CustomModels\AduInkasso\Bonitaetdatensatz
     */
    public function createNewScore(\Shopware\Models\Customer\Customer $customer = null)
    {
        if ($customer === null) {
            return false;
        }
        try {

            $isCompany = false;

            // ist Firma?
            if ($customer->getBilling()->getCompany() != '') {
                $score = 0;
                $info = 'Die Bonitätsabfrage von Firmenkunden ist zur Zeit nicht möglich.';
                $isCompany = true;
            } else {
                $billing = $customer->getBilling();

                try {
                    $version = Shopware::VERSION;
                    if (version_compare($version, '5.2.0', '<') ) {
                        $birthday = ($billing->getBirthday() && $billing->getBirthday() instanceof DateTime && $billing->getBirthday()->getTimestamp() > 0) ? $billing->getBirthday()->format('d.m.Y') : '';
                        
                        $q = Shopware()->Models()->getRepository('\Shopware\Models\Customer\Billing')->createQueryBuilder('b');
                        $q->where('b.id = :id')->setParameter('id', $customer->getBilling()->getId());
                        $q->select('b.number');
                        
                        $billingNumber = $q->getQuery()->getSingleScalarResult();                        
                    }else{
                        $birthday = ($customer->getBirthday() && $customer->getBirthday() instanceof DateTime && $customer->getBirthday()->getTimestamp(
                            ) > 0) ? $customer->getBirthday()->format('d.m.Y') : '';
                    }

                    $data = array(
                        '4', // Bonitätsprüfung
                        $billing->getFirstName(),
                        $billing->getLastName(),
                        $this->getStreetFromBilling($billing),
                        $this->getStreetNumberFromBilling($billing),
                        $billing->getZipCode(),
                        $billing->getCity(),
                        $billing->getPhone(),
                        $customer->getEmail(),
                        $birthday,
                        $billingNumber . ' ' . date('d.m.Y H:i'), // Referenznummer/Aktenzeichen (string)
                        false, // ist juristische Person (Boolean)
                        true // vollständige Informationen im XML Format (boolean)
                    );

                    //file_put_contents(__DIR__ . '/../tmp/score.txt', print_r($data, true), FILE_APPEND);

                    $return = $this->callClient('getConsumerCheck', $data);

                    try {
                        $xml = @new SimpleXMLElement($return);
                        Shopware()->Debuglogger()->addNotice('ADU', 'createNewScore Score XML - '.$xml->SCORE_WERT);
                        if (!isset($xml->SCORE_WERT)) {
                            throw new Exception('Score konnte nicht abgefragt werden.');
                        }

                        $score = (string) intval($xml->SCORE_WERT);

                        $score = $score / 10;

                        $info = $this->simpleToHtml($xml);
                    } catch (Exception $e) {
                        $score = 0;
                        $info = 'Fehler bei der Abfrage des Scores:<br />' . $return;
                    }
                } catch (Exception $e) {
                    $score = 0;
                    $info = 'Unbekanntes Problem mit der Schnittstelle';
                    Shopware()->Debuglogger()->addNotice('ADU', 'createNewScore Exception - '.$info);
                }
            }

            $newScore = new \Shopware\CustomModels\AduInkasso\Bonitaetdatensatz($customer->getId(), $score, new DateTime(), $info);
            $newScore->setIsCompany($isCompany);
            
            $doctrine = Shopware()->Models();
            $doctrine->persist($newScore);
            $doctrine->flush($newScore);
            Shopware()->Debuglogger()->addNotice('ADU', 'createNewScore Score ermittelt - '.$newScore->getScore());
            return $newScore;
        } catch (Exception $e) {
            return $e->getMessage();
        }

    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param SimpleXMLElement $elem
     * @return string
     */
    private function simpleToHtml(SimpleXMLElement $elem)
    {
        if ($elem->count() == 0) {
            return $elem->getName() . ' : ' . (string)$elem . '<br />';
        } else {
            $return = '';

            foreach ($elem as $sub) {
                $return .= '<br />' . $this->simpleToHtml($sub);
            }

            return $return;
        }
    }

    /**
     * Gibt die StreetName aus, unabhängig davon ob Shopware 4 oder 5 verwendet wird.
     * @param \Shopware\Models\Customer\Billing $billing
     */
    public function getStreetFromBilling(Shopware\Models\Customer\Billing $billing)
    {
        if (version_compare(Shopware::VERSION, '5.0.0', '>=')) {
            try {
                $tmp = $this->splitAddress($billing->getStreet());

                return $tmp['streetName'];

            } catch (\Exception $e) {
                return '';
            }

        } else { // VOR 5.0.0
            return $billing->getStreet();
        }
    }

    /**
     * Gibt die StreetNumber aus, unabhängig davon ob Shopware 4 oder 5 verwendet wird.
     * @param \Shopware\Models\Customer\Billing $billing
     */
    public function getStreetNumberFromBilling(Shopware\Models\Customer\Billing $billing)
    {
        if (version_compare(Shopware::VERSION, '5.0.0', '>=')) {
            try {
                $tmp = $this->splitAddress($billing->getStreet());

                return $tmp['houseNumber'];

            } catch (\Exception $e) {
                return '';
            }

        } else { // VOR 5.0.0
            return $billing->getStreetNumber();
        }
    }

    /**
     * Splittet die Straße aus Shopware 5 in array[streetName,houseNumber]
     * @param $address
     * @return array
     * @throws Exception
     */
    public function splitAddress($address)
    {
        $regex = '/\A\s*
        (?: #########################################################################
            # Option A: [<Addition to address 1>] <House number> <Street name>      #
            # [<Addition to address 2>]                                             #
            #########################################################################
            (?:(?P<A_Addition_to_address_1>.*?),\s*)? # Addition to address 1
        (?:No\.\s*)?
            (?P<A_House_number_1>\pN+[a-zA-Z]?(?:\s*[-\/\pP]\s*\pN+[a-zA-Z]?)*) # Street name
        \s*,?\s*
            (?P<A_Street_name_1>(?:[a-zA-Z]\s*|\pN\pL{2,}\s\pL)\S[^,#]*?(?<!\s)) # House number
        \s*(?:(?:[,\/]|(?=\#))\s*(?!\s*No\.)
            (?P<A_Addition_to_address_2>(?!\s).*?))? # Addition to address 2
        |   #########################################################################
            # Option B: [<Addition to address 1>] <Street name> <House number>      #
            # [<Addition to address 2>]                                             #
            #########################################################################
            (?:(?P<B_Addition_to_address_1>.*?),\s*(?=.*[,\/]))? # Addition to address 1
            (?!\s*No\.)(?P<B_Street_name>\S\s*\S(?:[^,#](?!\b\pN+\s))*?(?<!\s)) # House number
        \s*[\/,]?\s*(?:\sNo\.)?\s+
            (?P<B_House_number>\pN+\s*-?[a-zA-Z]?(?:\s*[-\/\pP]?\s*\pN+(?:\s*[\-a-zA-Z])?)*|[IVXLCDM]+(?!.*\b\pN+\b))(?<!\s) # Street name
        \s*(?:(?:[,\/]|(?=\#)|\s)\s*(?!\s*No\.)\s*
            (?P<B_Addition_to_address_2>(?!\s).*?))? # Addition to address 2
        )
        \s*\Z/x';
        $result = preg_match($regex, $address, $matches);
        try {
            if ($result === 0) {
                throw new Exception('Address \'' . $address . '\' could not be splitted into street name and house number');
            } else {
                if ($result === false) {
                    throw new Exception('Error occured while trying to split address \'' . $address . '\'');
                }
            }
        } catch (Exception $e) {
            $streetName = '';
            $houseNumber = '';
            $all = explode('.',$address);
            if ( count($all) > 1 ) {
                $houseNumber = $all[count($all)-1];
                unset($all[count($all)-1]);
                $streetName = implode('.',$all).'.';
            }

            return array(
                    'additionToAddress1' => '',
                    'streetName' => $streetName,
                    'houseNumber' => $houseNumber,
                    'additionToAddress2' => ''
                );
        }
        if (!empty($matches['A_Street_name'])) {
            return array(
                'additionToAddress1' => $matches['A_Addition_to_address_1'],
                'streetName' => $matches['A_Street_name'],
                'houseNumber' => $matches['A_House_number'],
                'additionToAddress2' => $matches['A_Addition_to_address_2']
            );
        } else {
            return array(
                'additionToAddress1' => $matches['B_Addition_to_address_1'],
                'streetName' => $matches['B_Street_name'],
                'houseNumber' => $matches['B_House_number'],
                'additionToAddress2' => $matches['B_Addition_to_address_2']
            );
        }
    }
}