<?php

/**
 * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
 * Class Shopware_Components_AduInkassoBonitaet
 */
class Shopware_Components_AduInkassoBonitaet
{

    protected $config;
    protected $api;

    protected $debug = false;
    protected $checkFiles = true;
    
    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param Enlight_Config $config
     * @param Shopware_Components_AduInkassoApi $api
     */
    public function __construct(Enlight_Config$config,Shopware_Components_AduInkassoApi $api) {
        $this->config = $config;
        $this->api = $api;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param $msg
     */
    public function log($msg) {
        if ( !$this->debug ) {
            return;
        }
        
        // Cleanup
        // Einmal pro Session nach alten Dateien filtern
        if($this->checkFiles == true){
            $files = scandir(__DIR__.'/../tmp/');
            if(count($files) > 1){
                foreach($files AS $k => $v){
                    $file = __DIR__.'/../tmp/'.$v;
                    $time = filemtime($file);
                     if($time < (time() - (7*24*60*60))){
                         @unlink ($file);
                     }
                }
            }
            $this->checkFiles = false;
        }
        
        file_put_contents(__DIR__.'/../tmp/bonitaet_'.date('d.m.Y_H',time()).'.txt',date('Y.d.m.H.i.s').': '.$msg.PHP_EOL,FILE_APPEND);
    }

    /**
     * Ermittelt den Creditstand und liefert diesen zurück.
     */
    public function getCreditStand() {
        if ( !$this->getApi()->testConnection() ) {
            return '-';
        }
        return $this->getApi()->callClient('getUserCredits');
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return bool
     */
    public function isZahlartOkOnCheckout()
    {
        
        Shopware()->Debuglogger()->addNotice('ADU', 'isZahlartOkOnCheckout gestartet');
        $this->log('');
        $this->log('Zahlart wird überprüft');
        if ($this->getConfig()->checkout_pruefen != 1) {
            $this->log('Zahlart beim Checkout prüfen ist deaktiviert');
            return true; // wenn wir keinen automatischen Checkout-Update durchführen sollen
        }

        // herausfinden welche Zahlungsart verwendet wurde
        $session = Shopware()->Session();
        if (!isset($session['sOrderVariables']['sPayment']['id'])) {
            $this->log('Keine Zahlart angegeben.');
            return true; // keine Zahlungsart, kein einmischen
        }

        $paymentID = $session['sOrderVariables']['sPayment']['id'];

        $this->log("Zahlart ist #".$paymentID);

        // nutzt diese Zahlungsart das ADUINKASSO Riskmanagement
        $sSQL = "SELECT rule1, value1, rule2, value2 FROM s_core_rulesets WHERE paymentID = ? AND (`rule1` LIKE 'ADUINKASSO%' OR `rule2` LIKE 'ADUINKASSO%') ORDER BY id ASC";
        $version = Shopware::VERSION;
        if (version_compare($version, '5.1.0', '<')) {
            // Get all assigned rules
            $queryRules = Shopware()->Modules()->Admin()->sSYSTEM->sDB_CONNECTION->GetAll(
                    $sSQL,
                    array($paymentID)
            );
        }else{
            $queryRules = Shopware()->Db()->fetchAll(
                    $sSQL,
                    array($paymentID)
            );
        }        
        
        if (is_array($queryRules) && count($queryRules) <= 0) {
            $this->log('Zahlart nicht betroffen.');
            return true;
        }

        // Ab hier ist Payment betroffen

        $user = Shopware()->Modules()->Admin()->sGetUserData();
        $customerId = $user['additional']['user']['id'];

        /** @var \Shopware\Models\Customer\Customer $customer */
        $customer = Shopware()->Models()->find('Shopware\Models\Customer\Customer', $customerId);

        if ($customer !== null && $customer->getAttribute() !== null && $customer->getAttribute()->getAduInkassoBoniCheckExclude() == true) {
            $this->log('Kunde ist ausgenommen.');
            return true;
        }

        $groupRep = Shopware()->Models()->getRepository('Shopware\Models\Customer\Group');
        /** @var \Shopware\Models\Customer\Group $group */
        $group = $groupRep->findOneBy(array('key'=>$user['additional']['user']['customergroup']));
        if ($group) {
            $this->log('Kundengruppe ist'.$group->getName());
            if ($group->getAttribute() !== null && $group->getAttribute()->getAduInkassoBoniCheckExclude() == true) {
                $this->log('Kundengruppe ist ausgenommen.');
                return true;
            }
        }

        /** @var \Shopware\CustomModels\AduInkasso\Repository $rep */
        $rep = Shopware()->Models()->getRepository('Shopware\CustomModels\AduInkasso\Bonitaetdatensatz');

        $datensatz = $rep->getLastSuccessfullByCustomer($customerId);

        $today = new DateTime("now");

        if ($datensatz === null || (int)$datensatz->getAbfrageDatum()->diff($today)->format('%a') > $this->getConfig()->checkout_max_age) {
            if ( $datensatz === null ) {
                $this->log('Neuer Score wird angefordert, da bisher kein Score vorliegt.');
            } else {
                $this->log('Neuer Score wird angefordert, da der bisherige Score zu alt ist. vom '.$datensatz->getAbfrageDatum()->format('d.m.Y'));
            }
            // NEU
            $score = $this->getApi()->createNewScore($customer);

            if (version_compare($version, '5.2.0', '<') ) {
                $isCompany = $score->isCompany;
                $isValid = $score->isValid;
            }else{
                $isCompany = $score->isCompany();
                $isValid = $score->isValid();
            }
            $this->log("Score: ".$score->getScore());
            if (!$score->isValid()) {
                if ( $score !== null && $isCompany && !$isValid ) {
                    $this->log('Score wurde für Firma ermittelt. Default-Firmen-Score wird verwendet:'.$this->getConfig()->checkout_default_boniwert_company);
                    Shopware()->Session()->adu_inkasso_session_score = $this->getConfig()->checkout_default_boniwert_company;
                } else {
                    $this->log('Neuer Score fehlerhaft. Es wird Fehler-Score verwendet: '.$this->getConfig()->checkout_default_boniwert_error);
                    Shopware()->Session()->adu_inkasso_session_score = $this->getConfig()->checkout_default_boniwert_error;
                }
                $this->log('Vorläufiger Score-Wert wird für diese Session beibehalten! (seit Version 1.1.1)');
            } else {
                $this->log('Neuer Score liegt bei '.$score->getScore());
            }

        }

        // wir müssen erneut das riskmanagement testen
        $basket = Shopware()->Modules()->Admin()->sSYSTEM->sMODULES['sBasket']->sBASKET;

        $sperren = Shopware()->Modules()->Admin()->sManageRisks($paymentID, $basket, $user);
//        Shopware()->Session()->adu_inkasso_session_score = false;

        if ($sperren) {
            $this->log('Nach neuer Prüfung muss gesperrt werden.');
            return false;
        }

        $this->log('Zahlart muss nicht gesperrt werden.');

        return true;
    }


    /**
     * Inhaltlich übernommen von engine/core/class/sAdmin.php
     * Anpassung um es als Hook umzusetzen.
     *
     * @param $paymentID
     * @param $basket
     * @param $user
     * @param $subject
     * @return bool
     * @throws Exception
     */
    public function callsManageRisks($paymentID, $basket, $user, $subject)
    {
        Shopware()->Debuglogger()->addNotice('ADU','callsManageRisks wurde geladen');
        // nutzt diese Zahlungsart das ADUINKASSO Riskmanagement
        $sSQL = "SELECT rule1, value1, rule2, value2 FROM s_core_rulesets WHERE paymentID = ? ORDER BY id ASC";
        $version = Shopware::VERSION;
        if (version_compare($version, '5.1.0', '<')) {
            // Get all assigned rules
            $queryRules = Shopware()->Modules()->Admin()->sSYSTEM->sDB_CONNECTION->GetAll(
                    $sSQL,
                    array($paymentID)
            );
        }else{
            $queryRules = Shopware()->Db()->fetchAll(
                    $sSQL,
                    array($paymentID)
            );
        }        

        if (!is_array($queryRules) || empty($queryRules)) {
            return false;
        }

        // Get-User-Data 

        // Get Basket
        if (empty($basket)) {
            $session = Shopware()->Session();
            $basket = array(
                'content' => $session->sBasketQuantity,
                'AmountNumeric' => $session->sBasketAmount
            );
        }

        foreach ($queryRules as $rule) {

            if ($rule["rule1"] && !$rule["rule2"]) {
                $rule["rule1"] = "sRisk" . $rule["rule1"];
                if ($rule["rule2"]) {
                    $rule["rule2"] = "sRisk" . $rule["rule2"];
                }

                if($rule["rule1"] == 'sRiskADUINKASSO' || $rule["rule1"] == 'sRiskADUINKASSOPU' || $rule["rule1"] == 'sRiskADUINKASSOPUAB' || $rule["rule1"] == 'sRiskADUINKASSOPAB' || $rule["rule1"] == 'sRiskADUINKASSOPAA' || $rule["rule1"] == 'sRiskADUINKASSOMPB'){
                    $return = $this->sRiskOverallCheck($user, $basket, $rule["value1"], $rule["rule1"] );
                } else {
                    $return = $subject->executeParent(
                        $rule["rule1"],
                        array(
                            $user,
                            $basket,
                            $rule["value1"]
                        )
                    );
                }


                if (is_null($return)) {
                    throw new \Exception('Die Regel ' . $rule['rule1'] . ' existiert nicht.');
                }
                if ($return) {
                    return true;
                }
            } elseif ($rule["rule1"] && $rule["rule2"]) {
                $rule["rule1"] = "sRisk" . $rule["rule1"];
                if ($rule["rule2"]) {
                    $rule["rule2"] = "sRisk" . $rule["rule2"];
                }
                // AND
                if($rule["rule1"] == 'sRiskADUINKASSO' || $rule["rule1"] == 'sRiskADUINKASSOPU' || $rule["rule1"] == 'sRiskADUINKASSOPUAB' || $rule["rule1"] == 'sRiskADUINKASSOPAB' || $rule["rule1"] == 'sRiskADUINKASSOPAA' || $rule["rule1"] == 'sRiskADUINKASSOMPB'){
                    $return1 = $this->sRiskOverallCheck($user, $basket, $rule["value1"],$rule['rule1']);
                } else {
                    $return1 = $subject->executeParent(
                        $rule["rule1"],
                        array(
                            $user,
                            $basket,
                            $rule["value1"]
                        )
                    );
                }

                if($rule["rule2"] == 'sRiskADUINKASSO' || $rule["rule2"] == 'sRiskADUINKASSOPU' || $rule["rule2"] == 'sRiskADUINKASSOPUAB' || $rule["rule2"] == 'sRiskADUINKASSOPAB' || $rule["rule2"] == 'sRiskADUINKASSOPAA' || $rule["rule2"] == 'sRiskADUINKASSOMPB'){
                    $return2 = $this->sRiskOverallCheck($user, $basket, $rule["value2"],$rule['rule2']);
                } else {
                    $return2 = $subject->executeParent(
                        $rule["rule2"],
                        array(
                            $user,
                            $basket,
                            $rule["value2"]
                        )
                    );
                }
                if (is_null($return1)) {
                    throw new \Exception('Die Regel ' . $rule['rule1'] . ' existiert nicht.');
                }
                if (is_null($return2)) {
                    throw new \Exception('Die Regel ' . $rule['rule2'] . ' existiert nicht.');
                }
                if ($return1 && $return2) {
                    return true;
                }
            }
        }

        return false; // nicht sperren, default
    }

    
    /**
     * Methode zum prüfen  der Check+Colect Riskmanagement Regeln
     * @param $user
     * @param $basket
     * @param $rule
     * @param $switch
     * @return boolean
     */
    private function sRiskOverallCheck($user, $basket, $rule, $switch){
        $return = false;
        if ($switch == 'sRiskADUINKASSO') {
            $return = $this->sRiskADUINKASSO($user, $basket, $rule);
        }else{ 
            switch ($switch){
                case 'sRiskADUINKASSOPU':
                    $compare = 'Person unbekannt';
                    break;
                case 'sRiskADUINKASSOPUAB':
                    $compare = 'Person unbekannt/Anschrift bekannt';
                    break;
                 case 'sRiskADUINKASSOPAB':
                    $compare = 'Person und Anschrift bekannt';
                    break;
                case 'sRiskADUINKASSOPAA':
                    $compare = 'Person/Anschrift abweichend';
                    break;
                case 'sRiskADUINKASSOMPB':
                    $compare = 'Mehrere Personen bekannt';
                    break;
            }
            $this->log('Prüfe '.$switch);
            if(isset($user['additional']['user']['id']) && !empty($user['additional']['user']['id'])){
                Shopware()->Debuglogger()->addNotice('ADU','Kunde '.$user['additional']['user']['id']);
                $repository = Shopware()->Models()->getRepository('Shopware\CustomModels\AduInkasso\Bonitaetdatensatz');
                $datensatz = $repository->getLastByCustomer($user['additional']['user']['id']);

                $zusatz = '';
                if(isset($datensatz)){ 
                    $zusatz = $datensatz->getZusatzinformationen();
                }else {
                    Shopware()->Debuglogger()->addNotice('ADU', 'Keine Zusatzinformation');
                }

                if(strpos($zusatz, $compare) !== false){
                    Shopware()->Debuglogger()->addNotice('ADU', $compare.' im Riskmanagement gefunden');
                    $return =  true;
                }
            }
        }
        return $return;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return \Enlight_Config
     */
    public function getConfig()
    {
        return $this->config;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @return \Shopware_Components_AduInkassoApi
     */
    public function getApi()
    {
        return $this->api;
    }

    /**
     * Die ShopwareAG wünscht überall Kommentare, dies ist ein Kommentar.
     * @param $user
     * @param $basket
     * @param $value
     * @return bool
     */
    private function sRiskADUINKASSO($user, $basket, $value)
    {

        if (!isset($user['additional']['user']['id'])) {
            return false; // nicht sperren, soll ja erstmal alles auswählen dürfen
        }

        $session = Shopware()->Session();
        if (isset($session->adu_inkasso_session_score) && $session->adu_inkasso_session_score != false) {
            Shopware()->Debuglogger()->addNotice('ADU', 'Score - '.$session->adu_inkasso_session_score. ' - ' .$value);
            if ($session->adu_inkasso_session_score >= $value) {
                return true;
            }

            return false;
        }


        /** @var \Shopware\CustomModels\AduInkasso\Repository $rep */
        $rep = Shopware()->Models()->getRepository('Shopware\CustomModels\AduInkasso\Bonitaetdatensatz');

        $score = $rep->getLastSuccessfullByCustomer($user['additional']['user']['id']);
        #var_export($rep);
        #die();
        // get last score
        if ($score === null) {
            Shopware()->Debuglogger()->addNotice('ADU', 'Score ermittelt - NULL');
            return false; // nicht sperren
        }
        if ($score->getScore() >= $value) {
            Shopware()->Debuglogger()->addNotice('ADU', 'Score ermittelt - '.$score->getScore());
            return true; // sperren
        }
        Shopware()->Debuglogger()->addNotice('ADU', 'sRiskADUINKASSO geladen');

        return false; // nicht sperren
    }


}