//{namespace name=backend/AduInkasso/main}
//{block name="backend/AduInkasso/controller/main"}
Ext.define('Shopware.apps.AduInkasso.controller.Main', {

    extend: 'Ext.app.Controller',

    mainWindow: null,

    refs: [
        { ref: 'mainWindowTabs', selector: 'aduinkasso-view-window > tabpanel' },
        { ref: 'mainWindowGrid', selector: 'aduinkasso-view-window aduinkasso-view-grid' },
        { ref: 'detailFormular', selector: 'aduinkasso-view-window aduinkasso-view-detailformular' },
        { ref: 'gutschriftFormular', selector: 'aduinkasso-view-detailwindowgutschriften form' },
        { ref: 'zusatzkostenFormular', selector: 'aduinkasso-view-detailwindowzusatzkosten form' },
        { ref: 'akteEinstellenFormular', selector: 'aduinkasso-view-window-akte-einstellen form' },
        { ref: 'forderungenGrid', selector: 'aduinkasso-view-detailgridforderungen' }
    ],

    forderungsart_store: null,

    init: function () {
        var me = this;

        me.callParent(arguments);
        var connection = "{$aduinkasso_connection}";
        if (connection == 'open') {
            me.initNormal();
        } else {
            me.initError();
        }
    },

    initNormal: function () {
        var me = this;

        me.mainWindow = me.getView('Window').create({
            mainStore: me.subApplication.getStore('Akten').load()
        });

        me.mainWindow.show();

        me.control({
            'aduinkasso-view-window aduinkasso-view-grid': {
                editEntry: me.onGridEditEntry,
//                viewUser: me.onViewUser,
                itemdblclick: me.onGridEditEntryByDoubleClick,
                stopEntry: me.onGridStopEntry,
                textChanged: me.onGridTextChanged

            },
            'aduinkasso-view-detailformular button[action=saveForderung]': {
                click: me.onSaveForderung
            },
            'aduinkasso-view-detailformular button[action=abortSaveForderung]': {
                click: me.onAbortSaveForderung
            },
            'aduinkasso-view-detailwindowgutschriften button[action=saveGutschrift]': {
                click: me.onSaveGutschrift
            },
            'aduinkasso-view-detailformular aduinkasso-view-detailgridzusatzkosten': {
                addZusatzkosten: me.onAddZusatzkosten,
                deleteZusatzkosten: me.onDeleteZusatzkosten
            },
            'aduinkasso-view-window aduinkasso-view-detailgridgutschriften': {
                deleteGutschrift: me.onDeleteGutschrift,
                addGutschrift: me.onAddGutschrift
            },
            'aduinkasso-view-detailwindowzusatzkosten button[action=saveZusatzkosten]': {
                click: me.onSaveZusatzkosten
            },
            'aduinkasso-view-detailformular field[name=anrede]': {
                change: me.anredeHasChanged
            },
            'aduinkasso-view-window-akte-einstellen button[action=akteEinstellen]': {
                click: me.onAkteEinstellen
            },
            'aduinkasso-view-window-akte-einstellen field[name=einstellung_ID]': {
                change: me.onAkteEinstellenGrundChanged
            },
            'aduinkasso-view-window tabpanel': {
                tabchange: function (tabpanel, tab) {
                    if (tab.xtype == 'aduinkasso-view-grid') {
                        tab.getStore().reload();
                    }
                }
            }
        });
    },

    onGridTextChanged: function (field, searchString) {
        var me = this,
            store = field.up('grid').getStore();


        //scroll the store to first page
        store.currentPage = 1;

        //If the search-value is empty, reset the filter
        if (searchString.length === 0) {
            store.clearFilter();
        } else {
            //This won't reload the store
            store.filters.clear();
            //Loads the store with a special filter
            store.filter('filter', searchString);
        }
    },

    anredeHasChanged: function (field) {
        var rechtsform = field.up('form').down('field[name=rechtsform_ID]'),
            firma = field.up('form').down('field[name=firma]');

        var value = field.getValue();

        if (value == 'Firma') {
            rechtsform.enable();
            firma.enable();
        } else {
            rechtsform.disable();
            firma.disable();
        }
    },

    onAkteEinstellenGrundChanged: function (field) {
        var me = this,
            form = field.up('form');

        var disableAllFields = function () {
            var fields = form.query('field');
            for (var i in fields) {
                fields[i].disable();
            }
            field.enable();
        };

        disableAllFields();
        var showField = form.down('field[showOn=' + field.getValue() + ']');
        if (showField) {
            showField.enable();
        }
    },

    onAbortSaveForderung: function () {
        var me = this,
            tabs = me.getMainWindowTabs(),
            form = me.getDetailFormular(),
            mainWindowGrid = me.getMainWindowGrid();

        Ext.Msg.show({
            title: 'Bearbeitung abbrechen?',
            msg: 'Wollen Sie die Bearbeitung der Akte wirklich abbrechen. Bitte beachten Sie, dass eingestellte Gutschriften, Forderungen und Zusatzkosten bereits gespeichert sind. Nur die bearbeiteten Schuldnerdaten gehen verloren.',
            buttons: Ext.Msg.YESNOCANCEL,
            icon: Ext.Msg.QUESTION,
            fn: function (btn) {
                if (btn == 'yes') {
                    tabs.setActiveTab(mainWindowGrid);
                    form.tab.hide();
                }
            }
        });


    },

    onSaveForderung: function () {
        var me = this,
            form = me.getDetailFormular();

        var record = form.getRecord();

        if (!form.getForm().isValid()) {
            AduInkasso.Helper.formError();
            return;
        }

        form.getForm().updateRecord(record);

        record.save({
            callback: function (record, operation) {
                var t = operation.request.scope.reader.jsonData["message"];
                if (t == "1") {
                    AduInkasso.Helper.alert('Erfolg', 'Forderung wurde gespeichert.');
                } else {
                    AduInkasso.Helper.alert('Nachricht', t);
                }
            }
        });

    },

    onDeleteZusatzkosten: function (view, record) {
        var me = this,
            store = view.up('panel').getStore(),
            wind = view.up('window'),
            zusatzkosten_id = record.get('ID'),
            akte = view.up('form').getForm().getRecord();

        if (!akte.get('zusatzkosten_stornierbar')) {
            Ext.Msg.show({
                title: 'Die Zusatzkosten können zur Zeit nicht storniert werden.',
                msg: 'Die Zusatzkosten können zur Zeit nicht storniert werden.',
                buttons: Ext.Msg.OK,
                icon: Ext.Msg.QUESTION
            });
            return;
        }

        Ext.Msg.show({
            title: 'Zusatzkosten stornieren?',
            msg: 'Wollen Sie die Zusatzkosten wirklich stornieren?',
            buttons: Ext.Msg.YESNOCANCEL,
            icon: Ext.Msg.QUESTION,
            fn: function (btn) {
                if (btn == 'yes') {
                    wind.setLoading(true);

                    Ext.Ajax.request({
                        url: '{url action="cancelZusatzkosten"}',
                        params: { id: zusatzkosten_id, 'glaeubigerAktenzeichen': akte.get('glaeubigerAktenzeichen') },
                        success: function (response) {
                            var p = Ext.JSON.decode(response.responseText);
                            AduInkasso.Helper.alert('', p.message);
                            wind.setLoading(false);
                            store.reload();
                        }
                    });
                }
            }
        });
    },

    onDeleteGutschrift: function (view, record) {
        var me = this,
            store = view.up('panel').getStore(),
            wind = view.up('window'),
            gutschrift_id = record.get('ID'),
            akte = view.up('form').getForm().getRecord();

        if (!akte.get('gutschrift_stornierbar')) {
            Ext.Msg.show({
                title: 'Die Gutschrift kann zur Zeit nicht storniert werden.',
                msg: 'Die Gutschrift kann zur Zeit nicht storniert werden.',
                buttons: Ext.Msg.OK,
                icon: Ext.Msg.QUESTION
            });
            return;
        }

        Ext.Msg.show({
            title: 'Gutschrift stornieren?',
            msg: 'Wollen Sie die Gutschrift wirklich stornieren?',
            buttons: Ext.Msg.YESNOCANCEL,
            icon: Ext.Msg.QUESTION,
            fn: function (btn) {
                if (btn == 'yes') {
                    wind.setLoading(true);

                    Ext.Ajax.request({
                        url: '{url action="cancelGutschrift"}',
                        params: { id: gutschrift_id, 'glaeubigerAktenzeichen': akte.get('glaeubigerAktenzeichen') },
                        success: function (response) {
                            var p = Ext.JSON.decode(response.responseText);
                            AduInkasso.Helper.alert('', p.message);
                            wind.setLoading(false);
                            store.reload();
                        }
                    });
                }
            }
        });


    },
    onSaveGutschrift: function (button) {
        var me = this,
            form = this.getGutschriftFormular().getForm(),
            wind = button.up('window');

        if (!form.isValid()) {
            AduInkasso.Helper.formError();
            return;
        }

        var data = form.getValues();

        data.glaeubigerAktenzeichen = wind.glaeubigerAktenzeichen;

        // Speichern

        wind.setLoading(true);

        Ext.Ajax.request({
            url: '{url action="saveGutschrift"}',
            params: data,
            success: function (response) {
                var p = Ext.JSON.decode(response.responseText);
                AduInkasso.Helper.alert('', p.message);
                wind.setLoading(false);
                wind.destroy();
                // gutschriften aktualisieren
                var t = me.getDetailFormular().down('aduinkasso-view-detailgridgutschriften');
                t.getStore().reload();
            }
        });

//        record.save();
//
//        button.up('window').destroy();

    },

    onAkteEinstellen: function (button) {
        var me = this,
            wind = button.up('window'),
            form = me.getAkteEinstellenFormular().getForm();

        if (!form.isValid()) {
            AduInkasso.Helper.formError();
            return;
        }

        var data = form.getValues();

        data.glaeubigerAktenzeichen = wind.glaeubigerAktenzeichen;

        wind.setLoading(true);

        Ext.Ajax.request({
            url: '{url action="saveEinstellung"}',
            params: data,
            success: function (response) {
                var p = Ext.JSON.decode(response.responseText);
                AduInkasso.Helper.alert('', p.message);
                wind.setLoading(false);
                wind.destroy();
                me.getMainWindowGrid().getStore().reload();
            }
        });
    },

    onSaveZusatzkosten: function (button) {
        var me = this,
            form = this.getZusatzkostenFormular().getForm(),
            wind = button.up('window');

        if (!form.isValid()) {
            AduInkasso.Helper.formError();
            return;
        }

        var data = form.getValues();

        data.glaeubigerAktenzeichen = wind.glaeubigerAktenzeichen;

        // Speichern

        wind.setLoading(true);

        Ext.Ajax.request({
            url: '{url action="saveZusatzkosten"}',
            params: data,
            success: function (response) {
                var p = Ext.JSON.decode(response.responseText);
                AduInkasso.Helper.alert('', p.message);
                wind.setLoading(false);
                wind.destroy();
                // zusatzkosten aktualisieren
                var t = me.getDetailFormular().down('aduinkasso-view-detailgridzusatzkosten');
                t.getStore().reload();
            }
        });

    },

    initError: function () {
        Ext.create('Ext.window.Window', {
            title: 'ADU-Inkasso Fehler',
            layout: 'fit',

            width: 300,
            height: '30%',
            items: [
                {
                    xtype: 'component',
                    html: '<div style="color: #ff0000; margin: 10px;"><b>Es ist ein Fehler aufgetreten:</b><br /><br />Es konnte keine Verbindung zur ADU-Inkasso-Api hergestellt werden. Bitte überprüfen Sie Ihre Angaben im Plugin und versuchen Sie es erneut.</div>'
                }
            ]
        }).show();
    },

    onAddGutschrift: function (view) {
        var me = this,
            t = this.getView('DetailWindowGutschriften'),
            form = view.up('form'),
            forderungenStore = me.getForderungenGrid().getStore(),
            latestForderung = me._findLatestForderung(forderungenStore);

        t.create({
            latestForderungDate: ( latestForderung ) ? latestForderung.get('datumHauptforderung') : new Date(),
            glaeubigerAktenzeichen: form.getForm().getRecord().get('glaeubigerAktenzeichen')
        });
    },

    _findLatestForderung: function (store) {
        if (!store.first()) return undefined;
        var rec = store.first();
        store.each(function (record) {
            if (record.get('datumHauptforderung') > rec.get('datumHauptforderung')) {
                rec = record;
            }
        });
        return rec;
    },

    onAddZusatzkosten: function (button) {
        var me = this,
            t = this.getView('DetailWindowZusatzkosten'),
            form = button.up('form'),
            forderungenStore = me.getForderungenGrid().getStore(),
            latestForderung = me._findLatestForderung(forderungenStore);

        t.create({
            latestForderungDate: ( latestForderung ) ? latestForderung.get('datumHauptforderung') : new Date(),
            glaeubigerAktenzeichen: form.getForm().getRecord().get('glaeubigerAktenzeichen')
        });

    },

    onViewUser: function (record) {
        Shopware.app.Application.addSubApplication({
            name: 'Shopware.apps.Customer',
            action: 'detail',
            params: {
                customerId: 1//record.get("id")
            }
        });
    },

    getForderungsartStore: function () {
        var me = this;
        if (!me.forderungsart_store) {
            me.forderungsart_store = new Ext.data.JsonStore({
                // store configs
                storeId: 'aduinkasso-forderungsarten',

                proxy: {
                    type: 'ajax',
                    url: '{url action=getForderungsArten}',
                    reader: {
                        type: 'json',
                        root: 'data',
//                        idProperty: 'forderungsArt_ID',
                        totalProperty: 'total'
                    }
                },
                fields: ['forderungsArt_ID', 'schluessel', 'bezeichnung'],
                autoLoad: true
            });
        }
        return me.forderungsart_store;
    },

    onGridEditEntryByDoubleClick: function (grid, record) {
        var me = this;
        me.onGridEditEntry(record);
    },

    onGridStopEntry: function (record, store, view) {
        var me = this,
            wind = view.up('window');

        if (record.get('akte_loeschbar')) {
            Ext.Msg.show({
                title: 'Akte löschen?',
                msg: 'Wollen Sie die Akte wirklich löschen?',
                buttons: Ext.Msg.YESNOCANCEL,
                icon: Ext.Msg.QUESTION,
                fn: function (btn) {
                    if (btn == 'yes') {
                        wind.setLoading(true);

                        Ext.Ajax.request({
                            url: '{url action="deleteAkte"}',
                            params: { 'glaeubigerAktenzeichen': record.get('glaeubigerAktenzeichen') },
                            success: function (response) {
                                var p = Ext.JSON.decode(response.responseText);
                                AduInkasso.Helper.alert('', p.message);
                                wind.setLoading(false);
                                store.reload();
                            }
                        });
                    }
                }
            });
            return;
        }


        if (!record.get('akte_einstellbar')) {
            Ext.Msg.show({
                title: 'Die Akte kann nicht eingestellt werden.',
                msg: 'Die Akte kann nicht eingestellt werden.',
                buttons: Ext.Msg.OK,
                icon: Ext.Msg.QUESTION
            });
            return;
        }


        var wind = Ext.create('Shopware.apps.AduInkasso.view.WindowAkteEinstellen');
        wind.glaeubigerAktenzeichen = record.get('glaeubigerAktenzeichen');
    },

    onGridEditEntry: function (record) {
        var me = this,
            tabs = me.getMainWindowTabs(),
            form = me.getDetailFormular();

        //
        // FORDERUNGEN
        //
        var forderungsstore = me.subApplication.getStore('Forderungen');
        forderungsstore.getProxy().extraParams = { cn: record.get('glaeubigerAktenzeichen') };
        forderungsstore.load();

        var fordG = form.query('aduinkasso-view-detailgridforderungen')[0];
        fordG.forderungsarten = me.getForderungsartStore();
        fordG.reconfigure(forderungsstore);

        //
        // GUTSCHRIFTEN
        //
        var gutschriftenstore = me.subApplication.getStore('Gutschriften');
        gutschriftenstore.getProxy().extraParams = { cn: record.get('glaeubigerAktenzeichen') };
        gutschriftenstore.load();

        fordG = form.query('aduinkasso-view-detailgridgutschriften')[0];
        fordG.reconfigure(gutschriftenstore);

        //
        // ZUSATZKOSTEN
        //
        var zusatzkostenstore = me.subApplication.getStore('Zusatzkosten');
        zusatzkostenstore.getProxy().extraParams = { cn: record.get('glaeubigerAktenzeichen') };
        zusatzkostenstore.load();

        fordG = form.query('aduinkasso-view-detailgridzusatzkosten')[0];
        fordG.reconfigure(zusatzkostenstore);

        //
        // VORGAENGE
        //
        var vorgaengestore = me.subApplication.getStore('Vorgaenge');
        vorgaengestore.getProxy().extraParams = { cn: record.get('glaeubigerAktenzeichen') };
        vorgaengestore.load();

        fordG = form.query('aduinkasso-view-detailgridvorgaenge');
        for (var i in fordG) {
            fordG[i].reconfigure(vorgaengestore);
        }

        form.loadRecord(record);

        var buttons = form.down('[name=aktebuttons]');

        if ( record.get('akte_bearbeitbar')) {
            buttons.show();
        } else {
            buttons.hide();
        }

        form.setTitle('Akte: ' + record.get('vorname') + ' ' + record.get('name'));
        form.tab.show();
        tabs.setActiveTab(form);
    }

});
//{/block}