//{namespace name="backend/AduInkasso/main"}
//{block name="backend/AduInkasso/view/mein_formular"}
Ext.define('Shopware.apps.AduInkasso.view.DetailFormular', {

    extend: 'Ext.form.Panel',

    alias: 'widget.aduinkasso-view-detailformular',

    title: 'Akte',

    autoScroll: true,

    cls: 'shopware-form',

    layout: 'fit',

    initComponent: function () {
        var me = this;

        me.items = me.getTabs();

        me.callParent(arguments);
    },

    getTabs: function () {
        var me = this;

        return {
            xtype: 'tabpanel',
            layout: 'fit',
            items: [
                me.getAktenFormular(),
                me.getForderungenFormular(),
                me.getHistorie()
            ],
            dockedItems: [
                {
                    dock: 'bottom',
                    xtype: 'toolbar',
                    ui: 'shopware-ui',
                    name : 'aktebuttons',
                    items: [
                        '->',
                        {
                            xtype: 'button',
                            text: 'Abbrechen',
                            cls: 'secondary',
                            action: 'abortSaveForderung'
                        },
                        {
                            xtype: 'button',
                            text: 'Speichern',
                            cls: 'primary',
                            action: 'saveForderung'
                        }
                    ]
                }
            ]
        };

    },

    getForderungenFormular: function () {
        var me = this;

        return {
            xtype: 'container',
            title: 'Forderungen/Gutschriften/Zusatzkosten',
            overflowY: 'auto',
            layout: 'anchor',
            defaultAnchor: '80%',
            items: [
                me.getForderungenFormularForderungen(),
                me.getForderungenFormularGutschriften(),
                me.getForderungenFormularZusatzkosten()
            ]
        };

    },

    getForderungenFormularForderungen: function () {
        var me = this;

        return {
            xtype: 'fieldset',
            title: 'Forderungs-Daten',
            collapsed: false,
            collapsible: false,
            layout: 'fit',
            items: [
                {
                    xtype: 'aduinkasso-view-detailgridforderungen'
                }
            ]
        };
    },

    getForderungenFormularGutschriften: function () {
        var me = this;

        return {
            xtype: 'fieldset',
            title: 'Gutschriften und Zahlungen',
            collapsed: false,
            collapsible: false,
            layout: 'fit',
            items: [
                { xtype: 'aduinkasso-view-detailgridgutschriften' }
            ]
        };
    },

    getForderungenFormularZusatzkosten: function () {
        var me = this;

        return {
            xtype: 'fieldset',
            title: 'Zusätzlich angefallene Kosten',
            collapsed: false,
            collapsible: false,
            layout: 'fit',
            items: [
                { xtype: 'aduinkasso-view-detailgridzusatzkosten' }
            ]
        };
    },

    getHistorie: function () {

        return {
            xtype: 'fieldset',
            title: 'Historie',
            collapsed: false,
            collapsible: false,
            items: [
                { xtype: 'aduinkasso-view-detailgridvorgaenge' }
            ]
        };
    },

    getAktenFormular: function () {
        var me = this;

        return {
            xtype: 'container',
            title: 'Schuldner Akte',
            overflowY: 'auto',
            layout: 'anchor',
            defaultAnchor: '80%',
            items: [
                me.getAktenFormularVorgaenge(),
                me.getAktenFormularSchuldnerDaten()
            ]
        }
    },

    getAktenFormularVorgaenge: function () {
        return {
            xtype: 'fieldset',
            title: 'Vorgänge',
            collapsed: false,
            collapsible: true,
            items: [
                { xtype: 'aduinkasso-view-detailgridvorgaenge' }
            ]
        };
    },

    getAktenFormularSchuldnerDaten: function () {
        var me = this;

        return {
            xtype: 'fieldset',
            title: 'Schuldnerdaten',
            collapsed: false,
            collapsible: true,
            layout: 'column',
            items: [
                {
                    xtype: 'container',
                    columnWidth: .45,
                    layout: 'anchor',
                    items: me.getAktenFormularSchuldnerDatenLinks()
                },
                {
                    xtype: 'label',
                    html: '&nbsp;',
                    columnWidth: .1
                },
                {
                    xtype: 'container',
                    columnWidth: .45,
                    layout: 'anchor',
                    items: me.getAktenFormularSchuldnerDatenRechts()
                }
            ]
        };
    },

    getAktenFormularSchuldnerDatenLinks: function () {
        return [
            {
                xtype: 'combobox',
                anchor: '100%',
                fieldLabel: 'Anrede',
                name: 'anrede',
                displayField: 'name',
                valueField: 'abbr',
                value: 'AL',
                store: {
                    xtype: 'store',
                    fields: ['abbr', 'name'],
                    data: [
                        { "abbr": "Herr", "name": "Herr"},
                        { "abbr": "Frau", "name": "Frau"},
                        { "abbr": "Firma", "name": "Firma"},
                        { "abbr": "Sonstiges", "name": "Sonstiges"}
                    ]
                }
            },
            {
                xtype: 'combobox',
                anchor: '100%',
                fieldLabel: 'Rechtsform',
                name: 'rechtsform_ID',
                displayField: 'bezeichnung',
                valueField: 'ID',
                store: {
                    xtype: 'jsonstore',
                    proxy: {
                        type: 'ajax',
                        url: '{url action=getRechtsformen}',
                        reader: {
                            type: 'json',
                            root: 'data',
                            totalProperty: 'total'
                        }
                    },
                    fields: ['ID', 'bezeichnung'],
                    autoLoad: true
                }
            },
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'Firma', name: 'firma', allowBlank: true},
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'Name', name: 'name', allowBlank: true},
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'Vorname', name: 'vorname', allowBlank: true},
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'zu Hd.', name: 'zuHaenden', allowBlank: true},
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'Straße & Nr.', name: 'strasse', allowBlank: true},
            {
                xtype: 'fieldcontainer',
                anchor: '100%',
                layout: 'hbox',
                fieldLabel: 'PLZ & Ort',
                defaults: {
                    hideLabel: 'true'
                },
                items: [
                    {
                        xtype: 'combobox',
                        flex: 2,
                        fieldLabel: 'Staat',
                        name: 'staat',
                        displayField: 'name',
                        valueField: 'iso',
                        store: Ext.create('Shopware.apps.Base.store.Country',{
                            fields: ['name', 'iso'],
                            autoLoad: true
                        })
                    },
                    { xtype: 'textfield', flex: 2, fieldLabel: 'Postleitzahl', name: 'plz', allowBlank: true},
                    { xtype: 'textfield', flex: 3, fieldLabel: 'Ort', name: 'ort', allowBlank: true}
                ]
            },
            { xtype: 'textareafield', anchor: '100%', fieldLabel: 'Kommentar zu Schulder', name: 'kommentar', allowBlank: true},
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'Aktueller Arbeitgeber', name: 'arbeitgeber', allowBlank: true},
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'Name des Ehepartners', name: 'ehepartner', allowBlank: true},
            { xtype: 'aduinkasso-datefield', anchor: '100%', fieldLabel: 'Geburtstag', name: 'geburtstag', allowBlank: true},
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'ges. Vertreter', name: 'vertreter', allowBlank: true},
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'Kundennummer', name: 'glaeubigerAktenzeichen', allowBlank: true, disabled: true},
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'Insolvenz-AZ', name: 'insolvenzAZ', allowBlank: true}

        ];
    },
    getAktenFormularSchuldnerDatenRechts: function () {
        return [
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'Telefon', name: 'telefon', allowBlank: true},
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'Fax', name: 'fax', allowBlank: true},
            { xtype: 'textfield', anchor: '100%', fieldLabel: 'E-Mail', name: 'email', allowBlank: true},
            {
                xtype: 'fieldset',
                title: 'Bekannte Bankverbindung des Schuldners',
                layout: 'anchor',
                items: [
                    { xtype: 'textfield', anchor: '100%', fieldLabel: 'Bankleitzahl', name: 'blz', allowBlank: true},
                    { xtype: 'textfield', anchor: '100%', fieldLabel: 'Bank-Name', name: 'bank', allowBlank: true},
                    { xtype: 'textfield', anchor: '100%', fieldLabel: 'IBAN', name: 'iban', allowBlank: true},
                    { xtype: 'textfield', anchor: '100%', fieldLabel: 'BIC', name: 'bic', allowBlank: true},
                    { xtype: 'textfield', anchor: '100%', fieldLabel: 'Kontoinhaber', name: 'inhaber', allowBlank: true},
                    { xtype: 'textfield', anchor: '100%', fieldLabel: 'Kontonummer', name: 'kto', allowBlank: true}
                ]
            },
            {
                xtype: 'fieldset',
                title: 'Sonstiges',
                layout: 'anchor',
                items: [
                    { xtype: 'aduinkasso-datefield', name: 'evDatum', anchor: '100%', fieldLabel: 'Datum Eidesst. Versicherung', allowBlank: true },
                    { xtype: 'aduinkasso-datefield', name: 'haftbefehlDatum', anchor: '100%', fieldLabel: 'Datum Haftbefehl', allowBlank: true},
                    { xtype: 'aduinkasso-datefield', name: 'insolvenzDatum', anchor: '100%', fieldLabel: 'Datum Insolvenz', allowBlank: true}
                ]
            }
        ];
    }
})
;
//{/block}