//{block name="backend/AduInkasso/view/grid"}
Ext.define('Shopware.apps.AduInkasso.view.DetailGridGutschriften', {

    extend: 'Ext.grid.Panel',

    alias: 'widget.aduinkasso-view-detailgridgutschriften',

    store: { xtype: 'store', model: 'Shopware.apps.AduInkasso.model.Gutschrift' },

    features: [
        {
            ftype: 'summary'
        }
    ],

    initComponent: function () {
        var me = this;

        me.dockedItems = [
            {
                xtype: 'toolbar',
                dock: 'top',
                ui: 'shopware-ui',
                items: [
                    {
                        xtype: 'button',
                        iconCls: 'sprite-plus-circle',
                        text: 'Gutschrift oder Zahlung hinzufügen',
                        handler: function (view) {
                            me.fireEvent('addGutschrift', view);
                        }
                    }
                ]
            }
        ];

        me.columns = me.getColumns();

        me.addEvents('deleteGutschrift', 'addGutschrift');

        me.callParent(arguments);
    },

    getColumns: function () {
        var me = this;

        return [
            { header: 'Gutschriftnummer ', dataIndex: 'nummer', flex: 1 },
            { header: 'Rechnungsnummer', dataIndex: 'belegnummer', flex: 1 },
            { header: 'Datum', dataIndex: 'datum', flex: 1, renderer: Ext.util.Format.dateRenderer('d.m.Y') },
            { header: 'Art', dataIndex: 'gutschrift_ID',
                renderer: function (value) {
                    if (value == 1) return 'Gutschrift des Gläubigers';
                    return 'Zahlung des Schuldners';
                },
                summaryRenderer: function () {
                    return 'Zwischensumme:';
                }
            },
            { header: 'Betrag', dataIndex: 'betrag', flex: 2,
                summaryType: 'sum'
            }
            ,
            {
                header: 'Optionen',
                xtype: 'actioncolumn',
                width: 120,
                items: me.getActionColumn()
            }
        ];
    },

    getActionColumn: function () {
        var me = this;

        return [
            {
                iconCls: 'sprite-minus-circle-frame',
                tooltip: 'Gutschrift löschen',
                handler: function (view, rowIndex, colIndex, item, opts, record) {
                    me.fireEvent('deleteGutschrift', view, record);
                }
            }
        ];
    },

    getToolbar: function () {
        var me = this;

        me.toolbar = Ext.create('Ext.toolbar.Toolbar', {
            ui: 'shopware-ui',
            items: [
                '->',
                {
                    xtype: 'textfield',
                    name: 'searchfield',
                    emptyText: 'Suche',
                    cls: 'searchfield',
                    checkChangeBuffer: 700,
                    listeners: {
                        change: function (field, value) {
                            me.fireEvent('textChanged', field);
                        }
                    }
                }
            ]
        });

        return me.toolbar;

    },

    getPagingbar: function () {
        var me = this;

        return [
            {
                xtype: 'pagingtoolbar',
                displayInfo: true,
                store: me.store
            }
        ];
    }
});
//{/block}