//{namespace name="backend/mein_plugin/main"}
//{block name="backend/AduInkasso/view/grid"}
Ext.define('Shopware.apps.AduInkasso.view.Grid', {

    extend: 'Ext.grid.Panel',

    alias: 'widget.aduinkasso-view-grid',

    title: 'Akten-Übersicht',

    initComponent: function () {
        var me = this;

        me.columns = me.getColumns();

        me.tbar = me.getToolbar();

        me.bbar = me.getPagingbar();

        me.addEvents('editEntry', 'deleteEntry', 'textChanged');

        me.callParent(arguments);
    },


    getColumns: function () {
        var me = this;

        return [
            { header: 'AZ', dataIndex: 'aktenzeichen', flex: 1, sortable: true },
            { header: 'KundenNr', dataIndex: 'glaeubigerAktenzeichen', flex: 1, sortable: true },
            { header: 'Schuldner', sortable: true , flex: 1, dataIndex: 'name', renderer: function (value, meta, record) {
                return value + ', ' + record.get('vorname');
            }},
            { header: 'Firma', dataIndex: 'firma', flex: 1, sortable: true },
            {
                header: 'Betrag',
                dataIndex: 'betrag',
                flex: 1,
                sortable: true,
                renderer: function (value) {
                    return Ext.util.Format.currency(value) + ' €';
                }
            },
            { header: 'letzter Vorgang', sortable: true, flex: 1, dataIndex: 'letzteAenderung', renderer: function (value,meta,record) {
                return AduInkasso.Helper.vorgangRenderer(record,meta);
            }},
            {
                header: 'Status',
                dataIndex: 'stadium',
                width: 35,
                sortable: true,
                renderer: function (value, meta, record) {
                    return AduInkasso.Helper.statusRenderer(value, meta, record);
                }
            },
            {
                header: 'Offen',
                dataIndex: 'betragOffen',
                flex: 1,
                sortable: true,
                renderer: function (value) {
                    return Ext.util.Format.currency(value) + ' €';
                }
            },
            {
                header: 'Übernahmedatum',
                dataIndex: 'exportDatum',
                flex: 1,
                sortable: true,
                renderer: function (value) {
                    if (!value) {
                        return '<i>unbekannt</i>';
                    }
                    return Ext.util.Format.date(value);
                }
            },
            {
                header: 'Opt.',
                xtype: 'actioncolumn',
                width: 100,
                sortable: false,
                items: me.getActionColumn()
            }
        ];
    },

    getActionColumn: function () {
        var me = this;

        return [
            {
                iconCls: 'sprite-pencil',
                tooltip: 'Akte bearbeiten',
                handler: function (view, rowIndex, colIndex, item, opts, record) {
                    me.fireEvent('editEntry', record);
                }
            },
            {
                iconCls: 'sprite-gear--arrow',
                tooltip: 'Akte einstellen',
                handler: function (view, rowIndex, colIndex, item, opts, record) {
                    me.fireEvent('stopEntry', record, me.getStore(),view);
                }
            }
        ];
    },

    getToolbar: function () {
        var me = this;

        me.toolbar = Ext.create('Ext.toolbar.Toolbar', {
            ui: 'shopware-ui',
            items: [
                '->',
                {
                    xtype: 'textfield',
                    name: 'searchfield',
                    emptyText: 'Suche',
                    cls: 'searchfield',
                    checkChangeBuffer: 700,
                    listeners: {
                        change: function (field, value) {
                            me.fireEvent('textChanged', field, value);
                        }
                    }
                }
            ]
        });

        return me.toolbar;

    },

    getPagingbar: function () {
        var me = this;

        return [
            {
                xtype: 'pagingtoolbar',
                displayInfo: true,
                store: me.store
            }
        ];
    }
});
//{/block}