//{block name="backend/customer/view/detail/window" append}
Ext.define('Shopware.apps.Customer.view.detail.customer_preferences.Window', {


    override: 'Shopware.apps.Customer.view.detail.Window',

    detail_score_store: null,


    getTabs: function () {
        var me = this;


        var tabs = me.callParent(arguments);

        me.detail_score_store = Ext.create('Ext.data.Store', {
            autoLoad: true,
            fields: ['datum', 'score', 'zusatzinformationen'],
            proxy: {
                extraParams: {
                    'cid': me.record.get('id')
                },
                type: 'ajax',
                url: '{url controller="AduInkasso" action="getScoreList"}',
                reader: {
                    type: 'json',
                    root: 'data'
                }
            },
            listeners: {
                refresh: function (store) {
                    if (tabs[0] && tabs[0].items.get(0) && tabs[0].items.get(0).alias[0] == 'widget.customer-additional-panel') {
                        if (store.first()) {
                            tabs[0].items.get(0).AduInfoView.update(store.first().data);
                        }
                    }
                }
            }

        });


        if (!me.record.get('id')) {
            return tabs;
        }

        var tab = Ext.create('Ext.panel.Panel', {
            title: 'ADU-Inkasso-Bonitätsprüfung',
            name: 'adu-inkasso-boni-tab',
            layout: 'border',
            items: [
                {
                    region: 'center',
                    xtype: 'component',
                    cls: Ext.baseCSSPrefix + 'outer-customer-info-pnl',
                    name: 'adu-inkasso-customer-detail-score-field',
                    tpl: me.getAduInkassoRenderTpl(),
                    overflowY: 'scroll'
                },
                {
                    dockedItems: [
                        {
                            dock: 'top',
                            xtype: 'toolbar',
                            ui: 'shopware-ui',
                            items: [
                                {
                                    xtype: 'button',
                                    text: 'Scorewert abfragen',
                                    cls: 'primary',
                                    handler: function () {
                                        me.getNewScore();
                                    }

                                }
                            ]
                        }
                    ],
                    region: 'west',
                    xtype: 'grid',
                    name: 'adu-inkasso-score-detail-grid',
                    width: 200,
                    columns: [
                        { header: 'Datum', dataIndex: 'datum', sortable: false, flex: 1, renderer: Ext.util.Format.dateRenderer('d.m.Y') },
                        { header: 'Score', dataIndex: 'score', sortable: false, flex: 2,
                            renderer: function (value, a, t) {
                                return AduInkasso.Helper.getScoreLightRendered(value, t.get('datum'), a)
                            }
                        }
                    ],
                    store: me.detail_score_store
                }
            ]
        });
        me.boniTab = tab;
        tabs.push(tab);
        return tabs;
    },

    createTabPanel: function () {
        var me = this;
        me.callParent(arguments);


        if (me.openBoniTab && me.boniTab) {
            var panel = me.down('tabpanel');
            panel.setActiveTab(me.boniTab);
        }

    },

    getNewScore: function () {
        var me = this,
            cid = me.record.get('id');

        var isCompany = me.record.getBilling() && me.record.getBilling().first() && me.record.getBilling().first().get('company') != "";

        if (isCompany) {
            Ext.Msg.show({
                title: 'Scorewert kann nicht abgefragt werden.',
                msg: "Bei Firmenkunden ist eine Scorewertabfrage zur Zeit nicht möglich.",
                buttons: Ext.Msg.OK
            });
            return;
        }

        me.setLoading(true);
        Ext.Msg.show({
            title: 'Neuen Score-Wert abfragen?',
            msg: 'Soll eine neue, kostenpflichtige Abfrage gestartet werden?',
            buttons: Ext.Msg.YESNOCANCEL,
            icon: Ext.Msg.QUESTION,
            fn: function (btn) {
                if (btn != 'yes') {
                    me.setLoading(false);
                    return;
                }
                Ext.Ajax.request({
                    url: '{url controller="AduInkasso" action="getNewScore"}',
                    params: { "cid": cid },
                    success: function (response) {
                        var p = Ext.JSON.decode(response.responseText);
                        me.setLoading(false);
                        me.detail_score_store.reload();
                        AduInkasso.Helper.alert('', p.message);
                    }
                });
            }
        });
    },

    getAduInkassoRenderTpl: function () {
        return '{literal}<div class="media-info-pnl">' +
            '<div class="base-info">' +
            '<h3>Detailansicht</h3>' +
            '<p>Hierbei handelt es sich um stichtagsbezogene Daten.</p>' +
            '<p><strong>Datum der Abfrage:</strong> {datum:date("d.m.Y H:i")} Uhr</p>' +
            '<p><strong>Scorewert:</strong> {score}</p>' +
            '<p><strong>Zusätzliche Informationen:</strong></p>' +
            '<div>{zusatzinformationen}</div>' +
            '</div>' +
            '</div>{/literal}';
    }
});
//{/block}