//{block name="backend/index/application" append}
Ext.define('Shopware.apps.Index.aduinkCreditstandWidget.view.Main', {
    extend: 'Shopware.apps.Index.view.widgets.Base',
    alias: 'widget.aduink-creditstand',
    title: 'ADU-Inkasso: Creditstand',
    layout: 'column',


    visitorsStore: null,

    height: 120,

    /**
     * Initializes the widget.
     *
     * @public
     * @return void
     */
    initComponent: function () {
        var me = this;

        me.items = [];

        me.tools = [
            {
                type: 'refresh',
                scope: me,
                handler: me.refreshView
            }
        ];

        Ext.define('AduInk.AduInkassoCreditstand.Creditstand', {
            extend: 'Ext.data.Model',
            fields: [
                'creditCredits',
                'creditDate'
            ]
        });

        me.visitorsStore = Ext.create('Ext.data.Store', {
            model: 'AduInk.AduInkassoCreditstand.Creditstand',
            remoteFilter: true,
            autoLoad: true,
            clearOnLoad: false,

            proxy: {
                type: 'ajax',
                url: '{url controller="AduInkassoCreditstand" action="credits"}',
                reader: {
                    type: 'json',
                    root: 'data'
                }
            }
        });

        me.visitorsStore.load({
            callback: function () {
                me.createColumnContainers();

                me.createTaskRunner();
            }
        });

        me.callParent(arguments);
    },

    /**
     * Creates the necessary containers for the layout.
     *
     * @public
     * @return [array] array of Ext.container.Container's
     */
    createColumnContainers: function () {
        var me = this,
            stores = me.visitorsStore.first();


        me.dataView = Ext.create('Ext.view.View', {
            tpl: me.createVisitorsOnlineTemplate(),
            data: [
                {
                    credits: stores.get('creditCredits'),
                    dateFormatted: stores.get('creditDate')
                }
            ]
        });

        /** Left container */
        me.add(Ext.create('Ext.container.Container', {
            columnWidth: 1,
            height: '100%',
            items: [
                me.dataView
            ]
        }));
    },

    /**
     * Helper method which creates the template
     * for all current visitors in the shop.
     *
     * @public
     * @return { object } Ext.XTemplate
     */
    createVisitorsOnlineTemplate: function () {
        var me = this;

        return new Ext.XTemplate(
            '{literal}',
            '<tpl for=".">',
            '<div class="adu-credits-view">',
            '<div class="adu-main">',
            '<div class="adu-coins"></div>',
            '<div class="adu-credits">{credits} Credits</div>',
            '</div>',
            '<div class="adu-under">',
            'Zuletzt aktualisiert: {dateFormatted}',
            '</div>',
            '</div>',
            '</tpl>',
            '{/literal}'
        );
    },

    /**
     * Registers a new task runner to refresh
     * the store after a given time interval.
     *
     * @public
     * @return void
     */
    createTaskRunner: function () {
        var me = this;

        me.storeRefreshTask = Ext.TaskManager.start({
            scope: me,
            run: me.refreshView,
            interval: 300000
        });
    },

    /**
     * Helper method which will be called by the
     * task runner and when the user clicks the
     * refresh icon in the panel header.
     *
     * @public
     * @return void
     */
    refreshView: function () {
        var me = this;

        if (!me.visitorsStore) {
            return;
        }

        me.setLoading(true);

        me.visitorsStore.load({
            callback: function () {
                var stores = me.visitorsStore.first();

                me.dataView.update([
                    {
                        credits: stores.get('creditCredits'),
                        dateFormatted: stores.get('creditDate')
                    }
                ]);
                me.setLoading(false);
            }
        });
    }
});
//{/block}