//{block name="backend/order/view/list/list" append}
Ext.define('Shopware.apps.Order.view.list.List.adu_inkasso.List', {

    /**
     * Defines an override applied to a class.
     * @string
     */
    override: 'Shopware.apps.Order.view.list.List',

    /**
     * Overrides the getColumns function of the overridden ExtJs object
     * and inserts two new columns
     * @return
     */
    getToolbar: function () {
        var me = this;

        var toolbar = me.callParent(arguments);

        me.aduInkassoButton = Ext.create('Ext.button.Button',
            {
                iconCls: 'sprite-documents-stack',
                text: 'Akte in ADU-Inkasso erstellen',
                disabled: true,
                handler: function () {
                    me.aduInkassoButtonClick();
                }
            }
        );
        toolbar.insert(1, me.aduInkassoButton);
        return toolbar;
    },

    aduInkassoButtonClick: function () {
        var me = this;

        me.setLoading(true);
        Ext.Msg.show({
            title: 'Bestellungen als ADU-Inkasso Akten anlegen?',
            msg: 'Prüfen Sie vor der Übergabe der Forderung an ADU-Inkasso, ob alle Kosten, Gutschriften, und Informationen in der Forderung enthalten sind. Nach der Übergabe kann die Forderung erst nach Rückmeldung des Exportdatum wieder bearbeitet werden. Die übergebene Forderung muss fällig und unbestritten sein.',
            buttons: Ext.Msg.YESNOCANCEL,
            icon: Ext.Msg.QUESTION,
            fn: function (btn) {
                if (btn != 'yes') {
                    me.setLoading(false);
                    return;
                }
                // speihern
                var t = me.getSelectionModel().getSelection();
                var ids = [];
                for (var i in t) {
                    ids.push(t[i].get('id'));
                }
                // onSuccess
                // AJAX
                Ext.Ajax.request({
                    url: '{url controller="AduInkasso" action="transferOrders"}',
                    params: { "orders[]": ids },
                    success: function (response) {
                        var p = Ext.JSON.decode(response.responseText);
                        me.setLoading(false);
                        me.getStore().reload();
                        AduInkasso.Helper.alert('', p.message);
                    }
                });
            }
        });
    },

    getGridSelModel: function () {
        var me = this;

        var selectionModel = me.callParent(arguments);

        selectionModel.addListener('selectionchange', function (sm, selections) {
            if (me.aduInkassoButton !== null) {
                me.aduInkassoButton.setDisabled(selections.length === 0);
            }
        });

        return selectionModel;
    },

    getColumns: function () {
            var me = this;

            var columns = me.callParent(arguments);

            var columnSize = {
                header: 'Score',
                tooltip: 'Zusatzinformationen zum Score',
                dataIndex: 'adu_score_value',
                flex: 1,
                renderer: function (value, a, t) {
                    return AduInkasso.Helper.getScoreLightRendered(value, t.get('adu_score_abfrage_datum'),a)
                }
            };

            return Ext.Array.insert(columns, 8, [columnSize]);
        }
});
//{/block}